@@ -1,13 +1,12 @@
 //===---------------------- backtrace_test.cpp ----------------------------===//
 //
-//                     The LLVM Compiler Infrastructure
-//
-// This file is dual licensed under the MIT and the University of Illinois Open
-// Source Licenses. See LICENSE.TXT for details.
+// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
+// See https://llvm.org/LICENSE.txt for license information.
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 //
 //===----------------------------------------------------------------------===//
 
-// UNSUPPORTED: libcxxabi-no-exceptions
+// UNSUPPORTED: no-exceptions
 
 #include <assert.h>
 #include <stddef.h>
@@ -49,7 +48,7 @@ void call1(size_t* ntraced, bool do_throw) {
   call2(ntraced, do_throw);
 }
 
-int main() {
+int main(int, char**) {
   size_t throw_ntraced = 0;
   size_t nothrow_ntraced = 0;
 
