@@ -1,16 +1,19 @@
 //===-------------------------- test_aux_runtime_op_array_new.cpp ---------===//
 //
-//                     The LLVM Compiler Infrastructure
-//
-// This file is dual licensed under the MIT and the University of Illinois Open
-// Source Licenses. See LICENSE.TXT for details.
+// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
+// See https://llvm.org/LICENSE.txt for license information.
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 //
 //===----------------------------------------------------------------------===//
 
-// UNSUPPORTED: libcxxabi-no-exceptions
+// UNSUPPORTED: no-exceptions
+
+// ___cxa_throw_bad_array_new_length is re-exported from libc++ only starting
+// in macosx 10.15
+// XFAIL: use_system_cxx_lib && target={{.+}}-apple-macosx10.{{9|10|11|12|13|14}}
 
-#include <iostream>
 #include <cxxabi.h>
+#include <new>
 
 //  If the expression passed to operator new[] would result in an overflow, the
 //  allocation function is not called, and a std::bad_array_new_length exception
@@ -28,11 +31,10 @@ bool bad_array_new_length_test() {
     return false;
 }
 
-int main(int argc, char *argv []) {
+int main(int, char**) {
     int ret_val = 0;
 
     if ( !bad_array_new_length_test ()) {
-        std::cerr << "Bad array new length test failed!" << std::endl;
         ret_val = 1;
     }
 
